﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;

namespace EnemyPlacementTester
{
    // an enum for the different enemy types
    public enum eEnemy { virus, cell };

    public partial class frmEnemyPlacement : Form
    {
        // global variables
        CDiagram diagram;
        CItemCreator creator;

        Graphics g;
        eEnemy current;
        bool selecting = false;

        public frmEnemyPlacement()
        {
            InitializeComponent();

            // initialises the list and the graphics component
            diagram = new CDiagram();
            g = pnlScreen.CreateGraphics();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (MessageBox.Show("Are you sure you wish to exit?", "Closing", MessageBoxButtons.YesNo, MessageBoxIcon.Exclamation) != DialogResult.Yes)
            {
                e.Cancel = true;
            }
        }

        #region menu selection functions
        // File - new
        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            diagram.reset();

            nudEnemyX.Enabled = false;
            nudEnemyY.Enabled = false;

            pnlScreen.Refresh();
        }

        // File - open
        private void openToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                // sets the diagram equal to the new diagram, if it is a valid file
                try
                {
                    diagram = CDiagram.load(openFileDialog.FileName);
                }
                catch(Exception exc)
                {
                    MessageBox.Show(exc.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }

                // refreshes the drawing area
                pnlScreen.Refresh();
            }
        }

        // File - save
        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // saves the diagram if required
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    diagram.save(saveFileDialog.FileName);
                }
                catch
                {
                    MessageBox.Show("Failed to save data correctly.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        // File - export
        private void exportToolStripMenuItem_Click(object sender, EventArgs e)
        {
            // gets all the enemy export data
            List<List<string>> l = new List<List<string>>();
            l = diagram.getExportData();

            StreamWriter sw;
            string temp;

            // asks the user for the files to export the data to
            if (exportDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    for (int i = 0; i < l.Count; i++)
                    {
                        // alters the filename for each different type of enemy to have
                        // a seperate data file
                        temp = Regex.Replace(exportDialog.FileName, ".txt", l[i][0] + ".txt");

                        // creates a new stream writer object
                        sw = new StreamWriter(temp);

                        // checks to see if the current list is empty
                        // (besides the file extension)
                        if (l[i].Count > 1)
                        {
                            // writes all elements of the current list
                            // but not the file extension
                            for (int j = 1; j < l[i].Count; j++)
                            {
                                sw.WriteLine(l[i][j]);
                            }
                        }

                        // closes the stream
                        sw.Close();
                    }
                }
                catch(Exception exc)
                {
                    MessageBox.Show(exc.Message + "\nCannot export data.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
        
        // File - exit
        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }
        #endregion

        #region pnlScreen functions
        private void pnlScreen_Paint(object sender, PaintEventArgs e)
        {
            // draws the player at the specified position
            g.DrawImage(Properties.Resources.player_One, (int)nudX.Value - (Properties.Resources.player_One.Width / 2),
                (int)nudY.Value - (Properties.Resources.player_One.Height / 2), Properties.Resources.player_One.Width,
                Properties.Resources.player_One.Height);

            // draws everything in the diagram
            diagram.drawItems(g, cbVision.Checked, cbRepulsion.Checked);

            // ensures that valid selection is currently being made
            if (diagram.getSelection() != null)
            {
                nudEnemyX.Value = diagram.getSelection().Area.X +(diagram.getSelection().Sprite.Width / 2);
                nudEnemyY.Value = diagram.getSelection().Area.Y +(diagram.getSelection().Sprite.Height / 2);
            }
        }

        private void pnlScreen_MouseClick(object sender, MouseEventArgs e)
        {
            if (!selecting)
            {
                // creates a new enemy based on the current data
                diagram.Items.Add(creator.createItem(current, e.Location));
            }
            else
            {
                // searches to see if there is an enemy where the user clicked
                diagram.select(e.Location);
            }

            // enables or disables the enemy movement controls based upon
            // whether an enemy is selected or not
            if (diagram.isValidSelection())
            {
                nudEnemyX.Enabled = true;
                nudEnemyY.Enabled = true;
            }
            else
            {
                nudEnemyX.Enabled = false;
                nudEnemyY.Enabled = false;
            }

            // refreshes the drawing area
            pnlScreen.Refresh();
        }

        private void pnlScreen_MouseMove(object sender, MouseEventArgs e)
        {
            // checks to see if the user is still clicking within the current selection
            if (diagram.clickingWithin(e.Location) && e.Button == MouseButtons.Left)
            {
                // keeps the selected enemy on screen
                if (e.Location.X <= nudEnemyX.Maximum && e.Location.X >= nudEnemyX.Minimum &&
                    e.Location.Y <= nudEnemyY.Maximum && e.Location.Y >= nudEnemyY.Minimum)
                {
                    diagram.moveSelection(e.Location, true);
                    pnlScreen.Refresh();
                }
            }

            // sets the text of two labels to the mouse position and displays them
            lblXPos.Text = e.X.ToString();
            lblXPos.Visible = true;
            lblXText.Visible = true;
            lblYPos.Text = e.Y.ToString();
            lblYPos.Visible = true;
            lblYText.Visible = true;
        }

        private void pnlScreen_MouseLeave(object sender, EventArgs e)
        {
            // hides the co-ordinate display
            lblXText.Visible = false;
            lblXPos.Visible = false;
            lblYText.Visible = false;
            lblYPos.Visible = false;
        }
        #endregion

        #region co ordinate display
        // updates the player's X position
        private void nudX_ValueChanged(object sender, EventArgs e)
        {
            pnlScreen.Refresh();
        }

        // updates the player's Y position
        private void nudY_ValueChanged(object sender, EventArgs e)
        {
            pnlScreen.Refresh();
        }

        // moves the current selection to location of the two numeric up/downs
        private void btnUpdate_Click(object sender, EventArgs e)
        {
            // only updates the selection if it is valid
            if (diagram.isValidSelection())
            {
                diagram.moveSelection(new Point((int)nudEnemyX.Value, (int)nudEnemyY.Value), false);
                pnlScreen.Refresh();
            }
        }
        #endregion

        #region creating drawing objects
        // sets the cursor to selection mode when in the panel
        private void btnSelect_Click(object sender, EventArgs e)
        {
            selecting = true;
        }

        // deletes the current selection
        private void btnDelete_Click(object sender, EventArgs e)
        {
            if (diagram.CurrentSelection > -1)
            {
                diagram.Items.RemoveAt(diagram.CurrentSelection);
                diagram.CurrentSelection = -1;
                pnlScreen.Refresh();
            }
        }

        // sets the mouse to draw a virus when clicked in the panel
        private void btnVirus_Click(object sender, EventArgs e)
        {
            selecting = false;
            current = eEnemy.virus;
            creator = new CEnemyCreator();
            pnlScreen.Enabled = true;
        }

        // sets the mouse to draw a cell when clicked in the panel
        private void btnCell_Click(object sender, EventArgs e)
        {
            selecting = false;
            current = eEnemy.cell;
            creator = new CEnemyCreator();
            pnlScreen.Enabled = true;
        }

        // sets vision radii to be shown or not
        private void cbVision_CheckedChanged(object sender, EventArgs e)
        {
            pnlScreen.Refresh();
        }

        // sets the repulsion radii to be shown or not
        private void cbRepulsion_CheckedChanged(object sender, EventArgs e)
        {
            pnlScreen.Refresh();
        }
        #endregion
    }
}
